// cc-by 3.0 Austria, (c) 2010 Markus Loeberbauer
// http://ssw.jku.at/Teaching/Lectures/SpezialLVA/JavaFX/
package at.jku.ssw.worms.score.crypto;

import java.io.UnsupportedEncodingException;
import java.security.GeneralSecurityException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

public final class Cryptor {

    private static final String CIPHER = "AES";
    private static final byte[] KEY = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
        12, 13, 14, 15};
    private static final String CHARSET_NAME = "UTF-8";
    private static final Cipher ENCRYPT_CIPHER;
    private static final Cipher DECRYPT_CIPHER;

    static {
        ENCRYPT_CIPHER = createCipher(Cipher.ENCRYPT_MODE);
        DECRYPT_CIPHER = createCipher(Cipher.DECRYPT_MODE);
    }

    public static String encrypt(String secret) {
        String encSecret;
        try {
            byte[] secretBytes = secret.getBytes(CHARSET_NAME);
            byte[] encBytes = encrypt(secretBytes);
            encSecret = toHexString(encBytes);
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(Cryptor.class.getName()).log(Level.SEVERE, null,
                    ex);
            encSecret = "";
        }
        return encSecret;
    }

    public static String decrypt(String message) {
        String decMessage;
        byte[] encBytes = fromHexString(message);
        byte[] decBytes = decrypt(encBytes);
        try {
            decMessage = new String(decBytes, CHARSET_NAME);
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(Cryptor.class.getName()).log(Level.SEVERE, null,
                    ex);
            decMessage = "";
        }
        return decMessage;
    }

    private static String toHexString(byte[] encrypted) {
        StringBuilder sb = new StringBuilder();
        for (byte b : encrypted) {
            sb.append(String.format("%02x", b));
        }
        return sb.toString();
    }

    private static byte[] fromHexString(String message) {
        byte[] encrypted = new byte[message.length() / 2];

        for (int i = 0, j = 0; i < encrypted.length; i++, j += 2) {
            try {
                encrypted[i] = (byte) Integer.parseInt(message.substring(j,
                        j + 2), 16);
            } catch (NumberFormatException nfe) {
                log(nfe);
                return new byte[0];
            }
        }
        return encrypted;
    }

    private static byte[] encrypt(byte[] secret) {
        try {
            return ENCRYPT_CIPHER.doFinal(secret);
        } catch (GeneralSecurityException e) {
            log(e);
        }
        return new byte[0];
    }

    private static byte[] decrypt(byte[] message) {
        try {
            return DECRYPT_CIPHER.doFinal(message);
        } catch (GeneralSecurityException e) {
            log(e);
        }
        return new byte[0];
    }

    private static Cipher createCipher(int mode) {
        SecretKeySpec keySpec = new SecretKeySpec(KEY, CIPHER);
        Cipher cipher = null;
        try {
            cipher = Cipher.getInstance(CIPHER);
            cipher.init(mode, keySpec);
        } catch (GeneralSecurityException e) {
            log(e);
        }
        return cipher;
    }

    private static void log(Exception e) {
        Logger.getLogger(Cryptor.class.getName()).log(Level.SEVERE,
                e.getMessage(), e);
    }
}
